#include <AccelStepper.h>
#include <Wire.h> 
#include <LiquidCrystal_I2C.h>
                  //  addr, en,rw,rs,d4,d5,d6,d7,bl,blpol
LiquidCrystal_I2C lcd(0x20, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);  // Set the LCD I2C address

// Define some steppers and the pins the will use
AccelStepper stepper1; // Defaults to AccelStepper::FULL4WIRE (4 pins) on 2, 3, 4, 5
AccelStepper stepper2(AccelStepper::FULL4WIRE, 6, 7, 8, 9);
AccelStepper stepper3(AccelStepper::FULL4WIRE, 10, 11, 12, 13);

#define MAX_COUNT 5
int NRot = 0; //value of rotation
int NRot_tmp = 0;
bool lay_dir = true;
int lay_tmp = 0;
int lay_min = 0;
int lay_max = 3;
long X = 20;
char inc_b; // incoming serial data;
int But_key = 0;

int readButtons (int pin) 
// returns the button number pressed, or zero for none pressed 
// int pin is the analog pin number to read 
{
  int b,c = 0;
  c=analogRead (pin); // get the analog value 
  if (c>1000)
  {
    b=0; // buttons have not been pressed
  } else
  if (c>310 && c<390)
  {
    b=1; // button 1 pressed
  } else
  if (c>270 && c<300)
  {
    b=2; // button 2 pressed
  } else
  if (c>220 && c<250)
  {
    b=3; // button 3 pressed
  } else
  if (c>130 && c<180)
  {
    b=4; // button 4 pressed
  } else
  if (c<110)
  {
    b=5; // button 5 pressed
  }
  return b;
}

int16_t get_rot(){
   int16_t tmp=0;  //int16_t -32766 .. 32766
   uint8_t count=0;
   uint8_t b; 
   int8_t minus=1; 
   do{
      //wait for next byte
      while(!Serial.available()){delay(1);}
      b = Serial.read();
      if(count==0 && b== '-'){
         minus=-1;
      }else
      if(b>= '0' && b<='9'){
         tmp*=10;
         tmp+=(b-'0');
         count++;
      }else{ //non integer byte!
         return tmp*minus;
      }
   }while( count<MAX_COUNT );
   return tmp*minus;
}

void forward()
{
    stepper2.runToNewPosition(40);
    stepper2.setCurrentPosition(0);
    stepper3.runToNewPosition(20);;
    stepper3.setCurrentPosition(0);
    stepper2.runToNewPosition(-40);
    stepper2.setCurrentPosition(0);
    stepper3.runToNewPosition(-20);
    stepper3.setCurrentPosition(0);
}

void backward()
{
    stepper2.runToNewPosition(-40);
    stepper2.setCurrentPosition(0);
    stepper3.runToNewPosition(-20);
    stepper3.setCurrentPosition(0);
    stepper2.runToNewPosition(40);
    stepper2.setCurrentPosition(0);
    stepper3.runToNewPosition(20);
    stepper3.setCurrentPosition(0);
}

void laying()
{
    if (lay_dir == true && lay_tmp < lay_max || lay_dir == true && lay_tmp == lay_min || lay_dir == false && lay_tmp == lay_min)
    {
      stepper1.runToNewPosition(X);
      stepper1.setCurrentPosition(0);
      lay_tmp++;
      lay_dir = true;
      Serial.print('+');
    }
    else
    {
      stepper1.runToNewPosition(-X);
      stepper1.setCurrentPosition(0);
      lay_tmp--;
      lay_dir = false;
      Serial.print('-');
    }
}

/*
void laying()
{
    stepper1.runToNewPosition(X);
    stepper1.setCurrentPosition(0);
}
*/

void laying_b()
{
    stepper1.runToNewPosition(-X);
    stepper1.setCurrentPosition(0);
}

void laying_home()
{
  stepper1.disableOutputs();
  stepper2.disableOutputs();
  stepper3.disableOutputs();
}

void setup()
{
    lcd.begin (8, 2);
    pinMode (A0, INPUT); // sets analog pin for input 
    Serial.begin(9600);
    Serial.println("Enter 'f' or 'b' for dir then value for number of rotation");
    stepper1.setMaxSpeed(1000.0);
    //stepper1.setSpeed(1000);
    stepper1.setAcceleration(600.0);
    
    stepper2.setMaxSpeed(1000.0);
    stepper2.setAcceleration(600.0);
	
    stepper3.setMaxSpeed(1000.0);
    stepper3.setAcceleration(600.0);
}

void loop()
{
    But_key=readButtons (0);
    lcd.clear ();
    if (Serial.available())
    {
      delay(2);
      inc_b = Serial.read();
      //But_key = readButtons (5); 
      if (inc_b == 'f') //forward // || But_key > 0
      {
        NRot = get_rot();
        NRot_tmp = 0;
        while(NRot_tmp < NRot)
        {
          forward();
          laying();
          NRot_tmp++;
        }
        lay_dir = true;
        NRot = 0;
        Serial.flush();
        Serial.println("END");
      }
      else if (inc_b == 'b')  //backward
      {
        NRot = get_rot();
        NRot_tmp = 0;
        while(NRot_tmp < NRot)
        {
          backward();
          laying();
          NRot_tmp++;
        }
        lay_dir = true;
        NRot = 0;
        Serial.flush();
        Serial.println("END");
      }
      
      /*
      else if (inc_b[5] == 'l')  //laying
      {
      laying();
      Serial.flush();
      }
      */
      
      else if (inc_b == 'q')  //exit
      {
      laying_home();
      Serial.flush();
      }
    }
    //But_key=readButtons (0);
    else if ( But_key == 1)
    {
      stepper1.runToNewPosition(X);
      stepper1.setCurrentPosition(0);
      Serial.println(But_key);
      lcd.setCursor (0,0);
      lcd.print ("tuda");
      delay (500); // give the human time to read LCD
    }
    else if ( But_key == 2)
    {
      stepper1.runToNewPosition(-X);
      stepper1.setCurrentPosition(0);
      Serial.println(But_key);
      lcd.setCursor (0,0);
      lcd.print ("suda");
      delay (500); // give the human time to read LCD
    }
}


